<?php

?>
<?php

/**
 * Helper class for the cleanup feature
 *
 * It searches the media folder for un-used images
 *
 */
class CloudCoders_NetSuite_Helper_Cleanup extends Mage_Core_Helper_Abstract
{

    protected $result = array();
    protected $_mainTable;
    public $validDirectories = array();

    /**
     * Returns a full list of all sub folders in the media folder
     *
     * @param string $path media folder path
     * @return array sub directories
     */
    public function listDirectories($path)
    {

        if (is_dir($path)) {
            try {
                if ($currentDirectory = opendir($path)) {
                    while (($entry = readdir($currentDirectory)) !== false) {
                        if (preg_match('/^\./', $entry) != 1) { // skip the dot directory
                            if (is_dir($path . DS . $entry) && !in_array($entry, array('cache', 'watermark'))) {
                                $this->listDirectories($path . DS . $entry);
                            } elseif (!in_array($entry, array('cache', 'watermark', 'placheholder')) && (strpos($entry, '.') != 0)) {
                                $this->result[] = substr($path . DS . $entry, 21);
                            }
                        }
                    }
                    closedir($currentDirectory);
                }
            } catch (Exception $e) {
                Mage::Log($e->getMessage());
            }
        }

        return $this->result;
    }

    /**
     * Verifies that an image is not in use
     *
     * Once an image is not in use, it is added to
     * the table of un-used images
     *
     */
    public function compareList()
    {

        $images = Mage::getModel('netsuite/cleanup')->getCollection()->getImages();

        $productImagesPath = 'media' . DS . 'catalog' . DS . 'product';

        $directories = $this->listDirectories($productImagesPath);

        $model = Mage::getModel('netsuite/cleanup');
        foreach ($directories as $item) {
            try {
                $item = strtr($item, '\\', '/');
                if (!in_array($item, $images)) {
                    $validDirectories[]['filename'] = $item;
                    $model->setData(array('filename' => $item))->setId(null);
                    $model->save();
                }
            } catch (Zend_Db_Exception $e) {
                Mage::log($e->getMessage());
            } catch (Exception $e) {
                Mage::log($e->getMessage());
            }
        }
    }

}