<?php

?>
<?php

/**
 * Factory class for creating various Netsuite data array
 * from magento entities
 *
 */
class CloudCoders_NetSuite_Helper_Factory extends Mage_Core_Helper_Abstract
{

    /**
     * Creates a new Order Instance
     *
     * @param int $magentoOrderId Magento Order Id
     * @return \Order
     */
    public function newOrder($magentoOrderId)
    {
        $netSuiteAccount = trim(Mage::getStoreConfig('system/netsuite/netsuite_account'));
        $defaultOrderFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Order.php';
        $customOrderFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Order_' . $netSuiteAccount . '.php';


        if (file_exists($customOrderFile)) {
            $className = 'Order_' . $netSuiteAccount;
            require_once $defaultOrderFile;
            require_once $customOrderFile;
            $Order = new $className($magentoOrderId);
        } else {
            require_once $defaultOrderFile;
            $Order = new Order($magentoOrderId);
        }
        return $Order;
    }

    /**
     * Creates a new Customer Instance
     *
     * @param int $magentoCustomerId
     * @return \Customer
     */
    public function newCustomer($magentoCustomerId)
    {

        $netSuiteAccount = trim(Mage::getStoreConfig('system/netsuite/netsuite_account'));
        $defaultCustomerFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Customer.php';
        $customCustomerFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Customer_' . $netSuiteAccount . '.php';

        if (file_exists($customCustomerFile)) {
            $className = 'Customer_' . $netSuiteAccount;
            require_once $defaultCustomerFile;
            require_once $customCustomerFile;
            $Customer = new $className($magentoCustomerId);
        } else {
            require_once $defaultCustomerFile;
            $Customer = new Customer($magentoCustomerId);
        }
        return $Customer;
    }

      /**
     * Create a new Magento Customer from Netusuite cusomer data
     * 
     * @param array $netsuiteData
     * @return \NetsuiteCustomer
     */
    public function newNesuiteCustomer(array $netsuiteData)
    {
        $netSuiteAccount = trim(Mage::getStoreConfig('system/netsuite/netsuite_account'));
        $netsuiteCustomerFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'NetsuiteCustomer.php';
        $customNetsuiteCustomerFile = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Model' . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'NetsuiteCustomer_' . $netSuiteAccount . '.php';

        if (file_exists($customNetsuiteCustomerFile)) {
            $className = 'NetsuiteCustomer_' . $netSuiteAccount;
            require_once $netsuiteCustomerFile;
            require_once $customNetsuiteCustomerFile;
            $Customer = new $className($netsuiteData);
        } elseif (file_exists($netsuiteCustomerFile)) {
            require_once $netsuiteCustomerFile;
            $Customer = new NetsuiteCustomer($netsuiteData);
        }
        return $Customer;
    }

}