<?php

?>
<?php

/**
 * Rest API endpoint for marking an order as "shipped" from netsuite to magento
 *
 * This Class extends the Rest
 */
class CloudCoders_NetSuite_Model_Api2_Shipment_Rest_Admin_V1 extends CloudCoders_NetSuite_Model_Api2_Shipment
{

    /**
     * Handle PUT/POST action
     *
     * @param array $salesOrderData Sales order data from Netsuite
     *
     * @return string A JSON data
     */
    protected function _create($salesOrderData)
    {


        // Make sure we have a salesorder ID
        if (!isset($salesOrderData['salesorder_id'])) {
            $this->_critical('You need to pass the salesorder id: Invalid salesorder_id', Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
            return;
        }

        $result = array(
            'invoiced' => false,
            'shipped' => false,
            'completed' => false,
        );

        // API Request Arguement
        $salesOrderId = $salesOrderData['salesorder_id'];
        $comment = (isset($salesOrderData['comment'])) ? $salesOrderData['comment'] : 'Updated via cloud coders extended order API';
        $notifyCustomer = (isset($salesOrderData['notify'])) ? $salesOrderData['notify'] : false;

        // Current Order
        $salesOrder = Mage::getModel('sales/order');
        $order = $salesOrder->load($salesOrderId);

        // Add an invoice for this order
        if ($order->canInvoice() && !$order->hasInvoices()) {
            $currentInvoice = $order->prepareInvoice();
            $currentInvoice->pay();
            $currentInvoice->sendEmail();
            $currentInvoice->register();
            $currentInvoice->save();
            $result['invoiced'] = true;
        }

        // Add a shipment for this order
        if ($order->canShip() && !$order->hasShipments()) {
            $currentShipment = $order->prepareShipment();           
            $currentShipment->setOrder($order);      
            $currentShipment->register();         
            $currentShipment->save();

           $salesOrderData['item_fulfillment'] = json_decode($salesOrderData['item_fulfillment'], true);
            if (isset($salesOrderData['item_fulfillment']['package']) && $salesOrderData['item_fulfillment']['package']) {
                $packages = $salesOrderData['item_fulfillment']['package'];
                foreach ($packages as $currentPackage) {
                    $TrackModel = Mage::getModel('sales/order_shipment_track');
                    $TrackModel->setCarrierCode('custom')
                            ->setNumber($currentPackage['packagetrackingnumber'])
                            ->setTitle($currentPackage['packagedescr']);
                    $currentShipment->addTrack($TrackModel);
                }

                $currentShipment->save();
            }
            
            $currentShipment->sendEmail(true);
            $currentShipment->setEmailSent(true);
            $currentShipment->save();
            $result['shipped'] = true;
        }

        // Update the order status
        if (strtolower($order->getStatusLabel()) != Mage_Sales_Model_Order::STATE_COMPLETE) {
            $order->setData('state', Mage_Sales_Model_Order::STATE_COMPLETE);
            $order->setStatus(Mage_Sales_Model_Order::STATE_COMPLETE);
            $order->setStatusHistoryComment($comment);
            $order->addStatusHistoryComment($comment);
            $order->setIsCustomerNotified($notifyCustomer);
             Mage::app()->getStore()->setConfig('cloudcoders_disable_order_event', true);
            $order->save();
            $result['completed'] = true;
        }

        $result['salesorder_id'] = $salesOrderId;
        print json_encode($result);
    }

}