<?php

?>
<?php

require_once __DIR__ . DIRECTORY_SEPARATOR . 'Interface' . DIRECTORY_SEPARATOR . 'ICustomer.php';
require_once __DIR__ . DIRECTORY_SEPARATOR . 'Logger.php';

/**
 * Magento Customer to Netsuite Customer base class
 *
 */
class Customer extends Logger implements ICustomer
{

    private $magentoCustomerId = 0;
    private $Customer = null;
    private $states = array(
        "Australian Capital Territory ( ACT )" => "Australian Capital Territory",
        "New South Wales ( NSW )" => "New South Wales",
        "Northern Territory ( NT )" => "Northern Territory",
        "Queensland ( QLD )" => "Queensland",
        "South Australia ( SA )" => "South Australia",
        "Tasmania ( TAS )" => "Tasmania",
        "Victoria ( VIC )" => "Victoria",
        "Western Australia ( WA )" => "Western Australia",
    );

    /**
     * Constructs a new instance
     *
     * @param int $magentoCustomerId Customer Id
     */
    public function __construct($magentoCustomerId = 0)
    {
        $this->magentoCustomerId = $magentoCustomerId;
    }

    /**
     * Set the current customer Magento's Id
     *
     * @param int $id
     */
    public function setCustomerId($id)
    {
        $this->magentoCustomerId = $id;
    }

    /**
     * Returns the current customer Magento Object
     *
     * @return object Magento Customer
     */
    public function getMagentoCustomer()
    {

        if ($this->Customer === null && $this->magentoCustomerId != 0) {
            $this->Customer = Mage::getModel('customer/customer')->load($this->magentoCustomerId);
        }

        return $this->Customer;
    }

    /**
     * appends the customer addresses
     *
     * @param object $Customer Magento Customer
     * @return array this customer data as an array
     */
    protected function appendAddresses($Customer)
    {
        $defBillingAddressId = $this->readIfPresent($Customer->getDefaultBilling());
        $defShippingAddressId = $this->readIfPresent($Customer->getDefaultShipping());

        $address_list = array();
        foreach ($Customer->getAddresses() as $address) {

            $addr_entity_id = $address->getEntityId();

            $streetArr = $address->getStreet();

            $is_default_billing = false;
            if ($addr_entity_id == $defBillingAddressId) {
                $is_default_billing = true;
            }
            $is_default_shipping = false;
            if ($addr_entity_id == $defShippingAddressId) {
                $is_default_shipping = true;
            }

            $firstname = $this->readIfPresent($address->getFirstname());
            $middlename = $this->readIfPresent($address->getMiddlename());
            $lastname = $this->readIfPresent($address->getLastname());
            $company = $this->readIfPresent($address->getCompany());
            $street1 = "";
            if (count($streetArr) >= 1) {
                $street1 = $this->readIfPresent($streetArr[0]);
            }
            $street2 = "";
            if (count($streetArr) >= 2) {
                $street2 = $this->readIfPresent($streetArr[1]);
            }
            $city = $this->readIfPresent($address->getCity());
            $postcode = $this->readIfPresent($address->getPostcode());
            $region = $this->getStateAbbrivation($this->readIfPresent($address->getRegion()));
            $country_id = $this->readIfPresent($address->getCountryId());
            $telephone = $this->readIfPresent($address->getTelephone());

            $new_addr = array(
                "address_id" => $addr_entity_id,
                "is_default_billing" => $is_default_billing,
                "is_default_shipping" => $is_default_shipping,
                "firstname" => $firstname,
                "middlename" => $middlename,
                "lastname" => $lastname,
                "company" => $company,
                "street1" => $street1,
                "street2" => $street2,
                "city" => $city,
                "postcode" => $postcode,
                "region" => $region,
                "country_id" => $country_id,
                "telephone" => $telephone
            );

            array_push($address_list, $new_addr);
        }

        $middlename = $this->readIfPresent($Customer->getMiddlename());
        $gender = $this->readIfPresent($Customer->getGender());
        $dob = $this->readIfPresent($Customer->getDob());
        $taxvat = $this->readIfPresent($Customer->getTaxvat());
        $Group = Mage::getModel('customer/group');
        $Group->load($Customer->getGroupId());

        $data = array("customer_id" => $Customer->getEntityId(),
            "firstname" => $Customer->getFirstname(),
            "middlename" => $middlename,
            "lastname" => $Customer->getLastname(),
            "email" => $Customer->getEmail(),
            "gender" => $gender,
            "dob" => $dob,
            "taxvat" => $taxvat,
            "group_id" => $Customer->getGroupId(),
            'group_code' => $Group->getCode(),
            'store_id' => $Customer->getStore()->getId(),
            'store_name' => $Customer->getStore()->getName(),
            "address_list" => $address_list
        );
        return $data;
    }

    /**
     * Returns this customer data as an array
     *
     * This is the data that is send to Nesuite
     * @return array Customer data
     */
    protected function getBuildNetsuiteCustomer()
    {
        $Customer = $this->getMagentoCustomer();
        if ($Customer) {
            return $this->appendAddresses($Customer);
        }
        return array();
    }

    /**
     * Returns the specified Australia state Abbrivation
     *
     * @param string $state
     * @return string
     */
    protected function getStateAbbrivation($state = '')
    {
        if (array_key_exists($state, $this->states)) {
            return $this->states[$state];
        }
        return $state;
    }

    /**
     * An alias for getBuildNetsuiteCustomer
     *
     * @return array Customer data
     */
    public function toArray()
    {
        return $this->getBuildNetsuiteCustomer();
    }

    /**
     * Returns this customer data as a JSON string
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Trys to read a variable value
     *
     * @param mix $data
     * @return string value
     */
    protected function readIfPresent($data)
    {
        return ($data === null) ? "" : $data;
    }

}