<?php

?>
<?php

/**
 * Description of NetsuiteCustomer
 *
 * @author Jibao
 */
class NetsuiteCustomer
{

    protected $customerNetsuiteData = array();
    protected $MagentoCustomer;
    protected $netsuiteItemMagentoIdUpdateUrl;
    protected $isNewCustomer = true;

    public function __construct($netsuiteData = array())
    {
        $this->customerNetsuiteData = $netsuiteData;
        $this->MagentoCustomer = Mage::getModel('customer/customer');

        if ($netsuiteData && isset($netsuiteData['customer_magento_id']) && $netsuiteData['customer_magento_id']) {
            $this->MagentoCustomer->load($netsuiteData['customer_magento_id']);
            if (!$this->MagentoCustomer->getId() && isset($netsuiteData['email'])) {
                $storeId = (isset($netsuiteData['store_id'])) ? $netsuiteData['store_id'] : Mage::app()->getWebsite()->getId();
                $this->MagentoCustomer->setWebsiteId($storeId);
                $this->MagentoCustomer->loadByEmail($netsuiteData['email']);
                if ($this->MagentoCustomer->getId()) {
                    $this->prepareCustomerForUpdate($netsuiteData);
                }
            } else {
                $this->prepareCustomerForUpdate($netsuiteData);
            }
        } else {

            if (isset($netsuiteData['email'])) {
                $this->MagentoCustomer->setWebsiteId(Mage::app()->getWebsite()->getId());
                $this->MagentoCustomer->loadByEmail($netsuiteData['email']);

                if ($this->MagentoCustomer->getId()) {
                    $this->prepareCustomerForUpdate($netsuiteData);
                }
            }
        }

        $this->netsuiteItemMagentoIdUpdateUrl = Mage::getStoreConfig('system/netsuite/netsuite_item_id_script');
    }

    /**
     * Sets the customer NetSuite data
     * 
     * @param array $data
     * @return \NetsuiteCustomer
     */
    public function setNesuiteData(array $data)
    {
        $this->customerNetsuiteData = $data;
        $netsuiteData = $data;

        $this->MagentoCustomer = Mage::getModel('customer/customer');
        if ($netsuiteData && isset($netsuiteData['customer_magento_id']) && $netsuiteData['customer_magento_id']) {
            $this->MagentoCustomer->load($netsuiteData['customer_magento_id']);
            if (!$this->MagentoCustomer->getId() && isset($netsuiteData['email'])) {
                $this->MagentoCustomer->loadByEmail($netsuiteData['email']);
                if ($this->MagentoCustomer->getId()) {
                    $this->prepareCustomerForUpdate($netsuiteData);
                }
            } else {
                $this->prepareCustomerForUpdate($netsuiteData);
            }
            $this->isNewCustomer = false;
        } else {
            if (isset($netsuiteData['email'])) {
                $this->MagentoCustomer->setWebsiteId(Mage::app()->getWebsite()->getId());

                $this->MagentoCustomer->loadByEmail($netsuiteData['email']);
                if ($this->MagentoCustomer->getId()) {
                    $this->prepareCustomerForUpdate($netsuiteData);
                }
            }
        }
        return $this;
    }

    /**
     * Returns the customer NetSuite data
     * 
     * @return array
     */
    public function getNesuiteData()
    {
        return $this->customerNetsuiteData;
    }

    /**
     * Returns the Magento Customer created from the NetSuite data
     * 
     * @return Object Magento Customer
     */
    public function getBuildMagentoCustomer()
    {
        $this->getGeneralInformation();
        $this->addAddresses();
        $this->addCustomerPrices();
        return $this->MagentoCustomer;
    }

    /**
     * Save the Magento Customer that was created
     * 
     * @return Object Magento Customer
     */
    public function saveCustomer()
    {
        return $this->getBuildMagentoCustomer();
    }

    protected function addCustomerPrices()
    {
        $customerPricesEnabled = Mage::getStoreConfigFlag('customerprices/settings/isenable');

        if ($customerPricesEnabled) {
            // Delete existing time prices
            $this->deleteExistingPrices('group_price');
            $this->deleteExistingPrices('item_price');

            // Add prices
            $this->addPrices('group_price');
            $this->addPrices('item_price');
        }
    }

    /**
     * Updates the Magento Customer information from the NetSuite data
     */
    protected function getGeneralInformation()
    {

        //  store id
        $storeId = Mage::app()->getWebsite()->getId();

        if (isset($this->customerNetsuiteData['store_id']) && $this->customerNetsuiteData['store_id']) {
            $storeId = $this->customerNetsuiteData['store_id'];
        }
        $this->MagentoCustomer->setStoreId($storeId);
        $this->MagentoCustomer->website_id = ($storeId);


        // group Id
        $groupId = 1; // General group
        if (isset($this->customerNetsuiteData['customer_group']) && $this->customerNetsuiteData['customer_group']) {
            $groupId = $this->customerNetsuiteData['customer_group'];
        }
        $this->MagentoCustomer->setData('group_id', $groupId);

        // Prefix
        if (isset($this->customerNetsuiteData['salutation'])) {
            $this->MagentoCustomer->setData('prefix', $this->customerNetsuiteData['salutation']);
        }

        // First Name
        if (isset($this->customerNetsuiteData['firstname'])) {          
            $this->MagentoCustomer->setFirstname($this->customerNetsuiteData['firstname']);
        }else{
            if(isset($this->customerNetsuiteData['companyname'])){    
                 $this->MagentoCustomer->setFirstname($this->customerNetsuiteData['companyname']);
            }
        }

        // Last Name
        if (isset($this->customerNetsuiteData['lastname'])) {
            $this->MagentoCustomer->setLastname($this->customerNetsuiteData['lastname']);
        }else{
            $this->MagentoCustomer->setLastname(' ');
        }

        // Email
        if ($this->customerNetsuiteData['email']) {
            $this->MagentoCustomer->setEmail($this->customerNetsuiteData['email']);
        }

        // Password
        if (!$this->isNewCustomer) {
            $this->MagentoCustomer->setPassword($this->MagentoCustomer->generatePassword(10));
            $this->MagentoCustomer->setConfirmation(null);
            $this->MagentoCustomer->sendNewAccountEmail();
        }
        try {
            $this->save();
            $this->sendMagentoId();
        } catch (Exception $ex) {
            Mage::log($ex->getMessage());
        }
    }

    /**
     * Update Magento Customer addresses from NetSuite data
     */
    protected function addAddresses()
    {


        $addresses = array();
        if (isset($this->customerNetsuiteData['addressbook'])) {
            $addresses = $this->customerNetsuiteData['addressbook'];
        }
        
        foreach ($addresses as $currentAddress) {
           
            if(isset($currentAddress['addressbookaddress'])){
               $temp = $currentAddress['addressbookaddress'];
               $temp['defaultbilling'] = $currentAddress['defaultbilling'];
               $temp['defaultshipping'] = $currentAddress['defaultshipping'];
               $temp['phone'] = (isset($currentAddress['addressbookaddress']['addrphone']))?$currentAddress['addressbookaddress']['addrphone']:'1111111111';
               $currentAddress = $temp;
            }

            $builtAddress = array();
            $CustomerAddresses = Mage::getModel('customer/address');
            if (isset($currentAddress['addressee'])) {

                // First and Last Name
                $exploadedFullName = explode(' ', $currentAddress['addressee']);
                $builtAddress['firstname'] = 'unknown';
                $builtAddress['lastname'] = 'unknown';
                if (isset($exploadedFullName[0])) {
                    $builtAddress['firstname'] = $exploadedFullName[0];
                    unset($exploadedFullName[0]);
                }

                if (!empty($exploadedFullName)) {
                    $builtAddress['lastname'] = implode(' ', $exploadedFullName);
                }

                // Street Address
                $builtAddress['street'] = array();

                // Street Address 1
                $builtAddress['street'][0] = (isset($currentAddress['addr1'])) ? $currentAddress['addr1'] : 'unknown';

                // Stret address 2 and 3
                $builtAddress['street'][1] = (isset($currentAddress['addr2'])) ? $currentAddress['addr2'] . '&nbsp;&nbsp;' : '';
                $builtAddress['street'][1] .= (isset($currentAddress['addr3'])) ? $currentAddress['addr3'] : '';

                // city
                $builtAddress['city'] = (isset($currentAddress['city'])) ? $currentAddress['city'] : 'unknown';
                // country id
                if (isset($currentAddress['country']) && is_array($currentAddress['country'])) {
                    $builtAddress['country_id'] = (isset($currentAddress['country']['internalid'])) ? $currentAddress['country']['internalid'] : 'AU';
                } else {
                    $builtAddress['country_id'] = 'AU';
                }

                // Rigion id / State Code
                if (isset($currentAddress['state']) && isset($builtAddress['country_id'])) {
                    $RegionModel = Mage::getModel('directory/region')->loadByCode($currentAddress['state'], $builtAddress['country_id']);
                    if ($RegionModel->getId()) {
                        $builtAddress['region_id'] = $RegionModel->getId();
                    }
                }
                // Region / State Display Name
                $builtAddress['region'] = (isset($currentAddress['displaystate'])) ? $currentAddress['displaystate'] : '';
                $builtAddress['state'] = $builtAddress['region'];

                // postcode
                $builtAddress['postcode'] = (isset($currentAddress['zip'])) ? $currentAddress['zip'] : '';

                // telephone
                $builtAddress['telephone'] = (isset($currentAddress['phone'])) ? $currentAddress['phone'] : '1111111111'; // Magento require a phone number

                $CustomerAddresses->setData($builtAddress);

                // default billing
                $CustomerAddresses->setIsDefaultBilling($currentAddress['defaultbilling']);

                // shipping billing
                $CustomerAddresses->setIsDefaultShipping($currentAddress['defaultshipping']);

                // Save
                $CustomerAddresses->setCustomerId($this->MagentoCustomer->getId());
                $CustomerAddresses->setSaveInAddressBook('1');
                try {
                    $CustomerAddresses->save();
                } catch (Exception $ex) {
                    Mage::log($ex->getMessage());
                }
            }
        }
    }

    /**
     * Updates the specified magento customer in netsuite
     *
     */
    protected function sendMagentoId()
    {
        $url = $this->netsuiteItemMagentoIdUpdateUrl;

        $id = false;
        if (isset($this->customerNetsuiteData['contactroles']) && is_array($this->customerNetsuiteData['contactroles'])) {
            $id = $this->customerNetsuiteData['contactroles'][0]['contact']['internalid'];
        }

        // When nesuite internal id does not exist
        if (!$id) {
            return;
        }

        // fields
        $url .= '&record_type=' . $this->customerNetsuiteData['recordtype'];
        $url .= '&item_id=' . $id;
        $url .= '&magento_id=' . $this->MagentoCustomer->getId();
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        try {
            curl_exec($ch);
        } catch (Exception $ex) {
            Mage::log($ex->getMessage());
        }
    }

    /**
     * Persist Magento Customer data to database
     */
    protected function save()
    {

        // Prevents the customer save event observer from
        // handling this particular save
        Mage::app()->getStore()->setConfig('cloudcoders_disable_customer_event', true);
        $this->MagentoCustomer->save();
    }

    /**
     * Prepares an existing Magento Customer for update
     * 
     * @param array $netsuiteData NetSuite customer data
     */
    protected function prepareCustomerForUpdate($netsuiteData)
    {
        // First Delete existing addresses
        foreach ($this->MagentoCustomer->getAddresses() as $CurrentAddress) {
            $CurrentAddress->setIsDefaultBilling(false);
            $CurrentAddress->setIsDefaultShipping(false);
            $CurrentAddress->delete();
        }

        $Id = $this->MagentoCustomer->getId();
        $this->MagentoCustomer = Mage::getModel('customer/customer'); // Creates a new object to prevent Magento caching
        if ($Id) {
            $this->MagentoCustomer->load($Id);
        }
        $this->MagentoCustomer->getAddresses();
        $this->isNewCustomer = ($Id) ? true : false;
    }

    protected function deleteExistingPrices($index)
    {

        $customerNetsuiteData = $this->getNesuiteData();
        if (isset($customerNetsuiteData[$index]) && !empty($customerNetsuiteData[$index])) {

            foreach ($customerNetsuiteData[$index] as $_item) {
                $CustomerPrices = Mage::getModel('customerprices/prices');
                $Product = Mage::getModel('catalog/product')->load($_item['magento_id']);

                if ($Product->getId()) {
                    $CustomerPrices->loadByCustomer($Product->getId(), $this->MagentoCustomer->getId(), $_item['min_quantity']);

                    if ($CustomerPrices && $CustomerPrices->getId()) {
                        $CustomerPrices->delete();
                    }
                }
            }
        }
    }

    protected function addPrices($index)
    {
        $customerNetsuiteData = $this->getNesuiteData();
        if (isset($customerNetsuiteData[$index]) && !empty($customerNetsuiteData[$index])) {

            foreach ($customerNetsuiteData[$index] as $_item) {
                $CustomerPrices = Mage::getModel('customerprices/prices');
                $Product = Mage::getModel('catalog/product')->load($_item['magento_id']);
                if ($Product->getId()) {
                    $CustomerPrices->loadByCustomer($Product->getId(), $this->MagentoCustomer->getId(), $_item['min_quantity'])
                            ->setCustomerId($this->MagentoCustomer->getId())
                            ->setCustomerEmail(trim($this->MagentoCustomer->getEmail()))
                            ->setProductId($_item['magento_id'])
                            ->setStoreId($this->MagentoCustomer->getWebsiteId())
                            ->setPrice($_item['unit_price'])
                            ->setQty($_item['min_quantity'])
                            ->setSpecialPrice($_item['unit_price']);


                    $CustomerPrices->save();
                    $CustomerPrices->setId(null);
                }
            }
        }
    }

}