<?php

?>
<?php

require_once __DIR__ . DIRECTORY_SEPARATOR . 'Interface' . DIRECTORY_SEPARATOR . 'IOrder.php';
require_once __DIR__ . DIRECTORY_SEPARATOR . 'Logger.php';

/**
 * Magento sales order to Nesuite sales oder base class
 *
 */
class Order extends Logger implements IOrder
{

    protected $magentoOrderId = 0;
    protected $netsuiteFields = array();
    protected $Order = null;
    protected $states = array(
        "Australian Capital Territory ( ACT )" => "Australian Capital Territory",
        "New South Wales ( NSW )" => "New South Wales",
        "Northern Territory ( NT )" => "Northern Territory",
        "Queensland ( QLD )" => "Queensland",
        "South Australia ( SA )" => "South Australia",
        "Tasmania ( TAS )" => "Tasmania",
        "Victoria ( VIC )" => "Victoria",
        "Western Australia ( WA )" => "Western Australia",
    );

    /**
     * Initialise a new Order Instance
     *
     * @param int $magentoOrderId Magento Sales Order Id
     */
    public function __construct($magentoOrderId = 0)
    {
        $this->magentoOrderId = $magentoOrderId;
    }

    /**
     * Sets the Magento Order Id for this Order
     *
     * @param int $magentoOrderId
     */
    public function setOrderId($magentoOrderId)
    {
        $this->magentoOrderId = $magentoOrderId;
    }

    /**
     * Returns the Magento Order associated with this order
     *
     * @return object Magento Sales Order
     */
    public function getMagentoOrder()
    {
        if ($this->Order === null && $this->magentoOrderId != 0) {
            $this->Order = Mage::getModel('sales/order')->load($this->magentoOrderId);
        }

        if ($this->Order && $this->Order->getId()) {
            return $this->Order;
        } else {
            return false;
        }
    }

    /**
     * Updates the Magento Sales Order Comment with the specified message
     *
     * @param string $message
     * @param boolean $notifyCustomer
     * @return \Order
     */
    public function updateOrderComment($message, $notifyCustomer = false)
    {
        $Order = $this->getMagentoOrder();
        if ($Order) {
            $Order->setStatus($Order->getStatus());
            $Order->addStatusHistoryComment($message, $Order->getStatus());
            $Order->setIsCustomerNotified($notifyCustomer);
            $Order->save();
        }
        return $this;
    }

    /**
     * Returns this order data as a JSON string
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Returns this order data as an Array
     *
     * @return array
     */
    public function toArray()
    {
        return $this->getBuildNetsuiteOrder();
    }

    /**
     * Returns this order data as an array
     *
     * @return array
     */
    protected function getBuildNetsuiteOrder()
    {
        $Order = $this->getMagentoOrder();
        if ($Order) {
            $this->netsuiteFields = $this->appendItems($Order);
            $this->netsuiteFields = $this->appendBillingAddress($Order);
            $this->netsuiteFields = $this->appendShippingAddress($Order);
            $this->netsuiteFields = $this->appendGeneralInformation($Order);
            return $this->netsuiteFields;
        }
        return array();
    }

    /**
     * Appends the order items information
     *
     * @param object $Order Magento Sales Order
     * @return array Item list
     */
    protected function appendItems($Order)
    {

        $OrderItems = array();
        foreach ($Order->getAllVisibleItems() as $CurrentOrderItem) {

            $productId = $CurrentOrderItem->getProductId();
            $productOptions = $CurrentOrderItem->getProductOptions();
            $simpleSku = (isset($productOptions['simple_sku'])) ? $productOptions['simple_sku'] : null;

            if ($simpleSku != null && $simpleSku != "") {
                //Now load the child
                $childProduct = Mage::getModel('catalog/product');

                $childProductId = $childProduct->getIdBySku($simpleSku);
                if ($childProductId) {
                    $productId = $childProductId;
                }
            }


            $price = $CurrentOrderItem->getBasePrice();
            $priceInclTax = $CurrentOrderItem->getBasePriceInclTax();
            $rowTotal = $CurrentOrderItem->getBaseRowTotal();
            $taxPercent = $CurrentOrderItem->getTaxPercent();
            $qty = $CurrentOrderItem->getQtyOrdered();
            $discountAmount = $CurrentOrderItem->getDiscountAmount();
            $hiddenTaxAmount = $CurrentOrderItem->getHiddenTaxAmount();
            $taxAmount = floatval(($taxPercent * $rowTotal) / 100.00);


            $CurrentOrderItemNesuiteData = array(
                "product_id" => $productId,
                "sku" => $simpleSku,
                "name" => $CurrentOrderItem->getName(),
                "price" => ($rowTotal - $taxAmount) / $qty,
                'old_price' => $price,
                "price_incl_tax" => $priceInclTax,
                "row_total" => $rowTotal,
                "tax_percent" => $taxPercent,
                "qty_ordered" => $qty,
                "discount_amount" => $discountAmount,
                "hidden_tax_amount" => $hiddenTaxAmount
            );
            $OrderItems[] = $CurrentOrderItemNesuiteData;
        }
        $this->netsuiteFields['item_list'] = $OrderItems;
        return $this->netsuiteFields;
    }

    /**
     * Appends the order billing address information
     *
     * @param object $Order Magento Sales Order
     * @return array Address segments
     */
    protected function appendBillingAddress($Order)
    {

        $billAddress = $Order->getBillingAddress();
        if ($billAddress) {
            $bill_email = $this->readIfPresent($billAddress->getEmail());
            $bill_streetArr = $billAddress->getStreet();
        } else {
            $bill_email = '';
        }

        if (!$bill_email) {
            $bill_email = $Order->getCustomerEmail();
        }

        $bill_street1 = "";
        if (count($bill_streetArr) >= 1) {
            $bill_street1 = $this->readIfPresent($bill_streetArr[0]);
        }
        $bill_street2 = "";
        if (count($bill_streetArr) >= 2) {
            $bill_street2 = $this->readIfPresent($bill_streetArr[1]);
        }

        $this->netsuiteFields += $this->buildAddress($Order->getBillingAddress(), 'bill');
        $this->netsuiteFields['bill_email'] = $bill_email;
        $this->netsuiteFields['bill_street1'] = $bill_street1;
        $this->netsuiteFields['bill_street2'] = $bill_street2;

        return $this->netsuiteFields;
    }

    /**
     * Trys to read a variable value
     *
     * @param mix $data
     * @return string
     */
    protected function readIfPresent($data)
    {
        return ($data === null) ? "" : $data;
    }

    /**
     * Appends this order shipping address
     *
     * @param string $Order Magento Sales Order
     * @return array Shipping address segments
     */
    protected function appendShippingAddress($Order)
    {

        $shipAddress = $Order->getShippingAddress();
        if ($shipAddress) {
            $ship_streetArr = $shipAddress->getStreet();
        } else {
            $ship_streetArr = '';
        }

        $ship_street1 = "";
        if (count($ship_streetArr) >= 1) {
            $ship_street1 = $this->readIfPresent($ship_streetArr[0]);
        }
        $ship_street2 = "";
        if (count($ship_streetArr) >= 2) {
            $ship_street2 = $this->readIfPresent($ship_streetArr[1]);
        }

        $this->netsuiteFields += $this->buildAddress($Order->getShippingAddress(), 'ship');

        $this->netsuiteFields['ship_street1'] = $ship_street1;
        $this->netsuiteFields["ship_street2"] = $ship_street2;

        return $this->netsuiteFields;
    }

    /**
     * Appends other/addtional information about this order
     *
     * @param object $Order Magento Sales Order
     * @return array order data
     */
    protected function appendGeneralInformation($Order)
    {
        $purchaseOrderNumber = $Order->getPayment()->getData('po_number');
        $shipping_carrier = '';
        $payment_method = '';
        $payment_method_code = '';
        $order_customer = array();
        $customer_id = $Order->getCustomerId();


        $ShippingCarrier = $Order->getShippingCarrier();
        $PaymentMethod = $Order->getPayment();

        if ($ShippingCarrier) {
            $shipping_carrier = $ShippingCarrier->getCarrierCode();
        }
        if ($PaymentMethod) {
            $payment_method = $PaymentMethod->getMethodInstance()->getTitle();
            $payment_method_code = $PaymentMethod->getMethodInstance()->getCode();
        }

        if ($customer_id) {
            $order_customer = Mage::helper('netsuite/factory')->newCustomer($customer_id)->toArray();
        }

        // Order comment
        $comment = '';
        $counter = 1;
        $Comments = $Order->getStatusHistoryCollection();
        foreach ($Comments as $CurrentComment) {
            if ($CurrentComment->getData('comment')) {
                $comment .= "($counter): " . $CurrentComment->getData('comment');
                $counter++;
            }
        }



        $data = array(
            "order_id" => $Order->getId(),
            "order_number" => $Order->getRealOrderId(),
            "customer_id" => $Order->getCustomerId(),
            "created_at" => $Order->getCreatedAt(),
            "remote_ip" => $Order->getRemoteIp(),
            "store_name" => $Order->getStoreName(),
            "store_id" => $Order->getStoreId(),
            "shipping_description" => $Order->getShippingDescription(),
            'shipping_carrier' => $shipping_carrier,
            "shipping_amount" => $Order->getShippingAmount(),
            "customer_group_id" => $Order->getCustomerGroupId(),
            "payment_method" => $payment_method,
            "payment_method_code" => $payment_method_code,
            'order_customer' => $order_customer,
            'comment' => $comment,
            'order_status' => $Order->getStatus(),
        );

        if ($purchaseOrderNumber) {
            $data['po_number'] = $purchaseOrderNumber;
        }
        return array_merge($this->netsuiteFields, $data);
    }

    /**
     * Returns the abbrivation for the sate specified
     *
     * @param string $state
     * @return string
     */
    protected function getStateAbbrivation($state = '')
    {
        if (array_key_exists($state, $this->states)) {
            return $this->states[$state];
        }
        return $state;
    }

    /**
     * 
     * @param type $AddressObj
     */
    protected function buildAddress($AddressObj, $type)
    {
        $address = array();
        if ($AddressObj) {
            $address = array(
                $type . '_streetArr' => $AddressObj->getStreet(),
                $type . '_firstname' => $this->readIfPresent($AddressObj->getFirstname()),
                $type . '_middlename' => $this->readIfPresent($AddressObj->getMiddlename()),
                $type . '_lastname' => $this->readIfPresent($AddressObj->getLastname()),
                $type . '_company' => $this->readIfPresent($AddressObj->getCompany()),
                $type . '_city' => $this->readIfPresent($AddressObj->getCity()),
                $type . '_postcode' => $this->readIfPresent($AddressObj->getPostcode()),
                $type . '_region' => $this->getStateAbbrivation($this->readIfPresent($AddressObj->getRegion())),
                $type . '_country_id' => $this->readIfPresent($AddressObj->getCountryId()),
                $type . '_telephone' => $this->readIfPresent($AddressObj->getTelephone()),
                $type . '_address_id' => $AddressObj->getCustomerAddressId(),
            );
        } else {
            $address = array(
                $type . '_streetArr' => '',
                $type . '_firstname' => '',
                $type . '_middlename' => '',
                $type . '_lastname' => '',
                $type . '_company' => '',
                $type . '_city' => '',
                $type . '_postcode' => '',
                $type . '_region' => '',
                $type . '_country_id' => '',
                $type . '_telephone' => '',
                $type . '_email' => '',
                $type . '_address_id' => '',
            );
        }

        return $address;
    }

    /**
     * Custom order class can override this method
     * to handle before save event for a sales order
     * 
     * @param Object $observer
     */
    public function beforeSave($observer)
    {
        
    }

    /**
     * Custom order class can override this method
     * to handle after save event for a sales order
     * 
     * @param Object $observer
     */
    public function afterSave($observer)
    {
        
    }

}