<?php

?>
<?php

/**
 * This class builds a Magento product from 
 * data sent from Netsuite
 */
class Product extends CloudCoders_Netsuite_Model_Simpleproduct
{

    protected $productData = array();
    protected $Product;

    /**
     * Class constructor
     * 
     * @param Object $MagentoProduct Magento Product
     * @param array $productData Netsuite Data array
     */
    public function __construct($MagentoProduct, $productData)
    {
        parent::__construct();
        $this->productData = $productData;
        $this->Product = $MagentoProduct;
        $this->setProductData($productData);
    }

    /**
     * Builds a Magento Product from Netsuite Item
     * 
     * @return Object Magento Product
     */
    public function getBuildProduct()
    {
        $Product = $this->Product;
        $productData = $this->productData;

        // Custom data
        $Product->setData('cc_netsuite_id', $productData['item_netsuite_id']);

        $productData['name'] = (isset($productData['name'])) ? $productData['name'] : ' ';
        $productData['description'] = (isset($productData['description'])) ? $productData['description'] : ' ';
        $productData['short_description'] = (isset($productData['short_description'])) ? $productData['short_description'] : ' ';
        $productData['weight'] = (isset($productData['weight'])) ? $productData['weight'] : '0.0';
        $productData['images'] = (isset($productData['images'])) ? $productData['images'] : array();
        $productData['related_items'] = (isset($productData['related_items'])) ? $productData['related_items'] : array();
        $productData['quantity'] = (isset($productData['quantity'])) ? $productData['quantity'] : '0';
        $productData['stock_message'] = (isset($productData['stock_message'])) ? $productData['stock_message'] : '';

        $Product->setSku($productData['sku']);
        $Product->setAttributeSetId('4');
        $Product->setTypeId('simple');
        $Product->setName($productData['name']);
        $Product->setDescription($productData['description']);
        $Product->setShortDescription($productData['short_description']);
        $Product->setData('cc_custom_stock_message', $productData['stock_message']);

        if (isset($productData['price_info']) && isset($productData['price_info']['price'])) {
            $price = $productData['price_info']['price'];
            $specialPrice = (isset($productData['price_info']['special_price']))?$productData['price_info']['special_price']: '';
            $specialPriceFromDate=  (isset($productData['price_info']['special_from_date']))?$productData['price_info']['special_from_date']:NULL;
            $specialPriceToDate=  (isset($productData['price_info']['special_to_date']))?$productData['price_info']['special_to_date']:NULL;
        } else {
            $price = 0.0;
            $specialPrice ='';
            $specialPriceFromDate = null;
            $specialPriceToDate  = null;
        }
        
        $Product->setPrice($price);
        $Product->setData('special_price',$specialPrice);
        $Product->setData('special_from_date',$specialPriceFromDate);
        $Product->setData('special_to_date',$specialPriceToDate);

        //Default Magento attribute
        $productData['weight'] = ($productData['weight']) ? $productData['weight'] : '0.0';
        $Product->setWeight($productData['weight']);
        $Product->setVisibility(Mage_Catalog_Model_Product_Visibility::VISIBILITY_BOTH);
        $Product->setStatus(1);

        $Product->setCreatedAt(strtotime('now'));

        // Images
        $Product = $this->setProductImages($Product, $productData['images']);

        // Related items
        $Product = $this->setProductRelatedItems($Product, $productData['related_items']);

        // Tax class
        $Product->setTaxClassId(2);

        // Url
        $Product->setData('url_key', Mage::getModel('catalog/product_url')->formatUrlKey($Product->getName()));

        $Product->setData('url_key_create_redirect', '1');
        $Product->setData('save_rewrites_history', false);
        $Product->setWebsiteIDs(array(1));

        Mage::getModel('Mage_Catalog_Model_Url')->refreshProductRewrite($Product->getId());

        // Select current store
        $Product = $this->setProductStoresAndCategory($Product, $productData);

        // Temando fields
        $Product = $this->setProductTemandoFields($Product, $productData);


        return $this->updateProductStockAndPrice($Product, $productData);
    }

    public function setProductStoresAndCategory($Product, $productData)
    {
        Mage::app()->setCurrentStore(Mage_Core_Model_App::ADMIN_STORE_ID);

        // Add product to categories
        $productCategoies = array();
        if (isset($productData['site_category']) && count($productData['site_category']) > 0) {

            foreach ($productData['site_category'] as $currentCategory) {
                $MagentoCategoryId = $this->getProductCategoryByNetsuiteId($currentCategory['item_netsuite_id']);
                // Item category exist
                if ($MagentoCategoryId) {
                    $productCategoies[] = $MagentoCategoryId->getId();
                }
            }
        }

        if ($productCategoies) {
            $Product->setCategoryIds($productCategoies);
        } else {
            // Add product to Site Root Category
            $RootCat = Mage::app()->getStore()->getRootCategoryId();
            $Product->setCategoryIds(array($RootCat));
        }

        return $this->setProductStores($Product, $productData);
    }

    /**
     * Adds prouct to stores
     * 
     * @param Object $Product Magento Product
     * @param array $productData Netsuite item data
     * @return Object Magento Product
     */
    public function setProductStores($Product, $productData)
    {
        $allStores = Mage::app()->getStores();
        if (isset($productData['stores']) && (count($allStores) > 1)) {
            $stores = (is_array($productData['stores'])) ? $productData['stores'] : json_decode($productData['stores'], true);
            $websites = array();
            foreach ($stores as $currentStoreId) {
                $websites[] = Mage::getModel('core/store')->load($currentStoreId)->getWebsiteId();
            }

            if ($websites) {
                $Product->setWebsiteIds($websites);
            }
        }
        return $Product;
    }

    /**
     * Update product images
     * 
     * @param Object $productObj Magento Product
     * @param array $imagesArray Array of the product images from NetSuite
     * @return Object Magento Product
     */
    public function setProductImages($productObj, $imagesArray = '')
    {
        if (empty($imagesArray)) {
            return $productObj;
        }

        // First Remove images
        Mage::app()->getStore()->setId(Mage_Core_Model_App::ADMIN_STORE_ID);
        if ($productObj->getId()) {
            $oldProduct = $productObj;
        } else {
            $oldProduct = $this->getProductByNetsuiteId($productObj->getData('cc_netsuite_id'));
        }

        if ($oldProduct) {

            $mediaApi = Mage::getModel("catalog/product_attribute_media_api");
            $items = $mediaApi->items($oldProduct->getId());
            if (is_array($items) && !empty($items)) {
                foreach ($items as $item) {
                    $mediaApi->remove($oldProduct->getId(), $item['file']);
                }
            }
        }

        $productObj->setMediaGallery(array('images' => array(), 'values' => array()));
        // Add product images
        foreach ($imagesArray as $imageIndex => $cur_image) {
            if (!$cur_image['id']) {
                continue;
            }

            $result = $this->getProductMedia($cur_image['id'], $cur_image['name'], $cur_image['type']);

            try {
                if ($result['filesize']) {
                    if (isset($imagesArray[$imageIndex + 1])) {
                        $productObj->addImageToMediaGallery($result['fullpath'], array(), false, false);
                    } else {
                        $productObj->addImageToMediaGallery($result['fullpath'], array('image', 'small_image', 'thumbnail'), false, false);
                    }
                }
            } catch (Exception $e) {
                Mage::log($e->getMessage());
            }
        }

        return $productObj;
    }

    /**
     * Appends related products to the current Magento Product
     * 
     * @param Object $productObj Magento Product
     * @param array $relatedItemsArray Netsuite Data
     * @return Object Magento Product
     */
    public function setProductRelatedItems($productObj, $relatedItemsArray)
    {
        $products = array();
        $position = 0;
        foreach ($relatedItemsArray as $curItem) {
            $position++;

            $curProduct = $this->getProductByNetsuiteId($curItem['item_netsuite_id']);
            if ($curProduct) {
                $products[$curProduct->getId()] = array(
                    'position' => $curProduct->getId(),
                );

                if ($curProduct->getName() == '&nbsp') {
                    $curProduct->setName('&nbsp;');
                    $curProduct->save();
                }
                if ($curProduct->getData('sku') != $curItem['sku']) {
                    $curProduct->setData('sku', $curItem['sku']);
                    $curProduct->save();
                }
            } else {
                $NewProduct = Mage::getModel('catalog/product')->loadByAttribute('sku', $curItem['sku']);
                if (!$NewProduct) {
                    $NewProduct = new Mage_Catalog_Model_Product();
                    $NewProduct->setStatus(1); // Enable
                    $NewProduct->setData('cc_netsuite_id', $curItem['item_netsuite_id']);
                    $NewProduct->setSku($curItem['sku']);
                    $NewProduct->setTypeId('simple');
                    $NewProduct->setAttributeSetId('4'); // Default attribute set
                    $NewProduct->setName('&nbsp;');
                    $NewProduct->setData('url_key', $curItem['item_netsuite_id']);
                    Mage::app()->setCurrentStore(Mage_Core_Model_App::ADMIN_STORE_ID);
                    $NewProduct->save();
                }
                $products[$NewProduct->getId()] = array(
                    'position' => $NewProduct->getId(),
                );
            }
        }

        if ($products) {
            $productObj->setRelatedLinkData($products);
        }
        return $productObj;
    }

    /**
     * Saves the current Product media to local files
     * 
     * @param int $mediaId Media file netsuite Id
     * @param string $mediaName Media file netsuite name
     * @param string $mediaType Media file content type
     * @return array with the fields [ur,fullpath,filenem,filesize]
     */
    public function getProductMedia($mediaId, $mediaName, $mediaType)
    {
        $result = array(
            'url' => Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA),
            'fullpath' => '',
            'filename' => '',
            'filesize' => '',
        );

        $mediaSubFolder = Mage::getStoreConfig('system/netsuite/netsuite_media_temp_folder');
        $url = $this->mediaUrl;
        $imageFolder = Mage::getBaseDir('media') . DIRECTORY_SEPARATOR . $mediaSubFolder . DIRECTORY_SEPARATOR;
        $ch = curl_init($url . $mediaId);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $data = curl_exec($ch);

        $fileExtension = $this->NetSuiteFileType->getExtension($mediaType);
        $filename = (strstr($mediaName, $fileExtension)) ? $mediaName : $mediaName . $fileExtension;
        $fullFileName = $imageFolder . $filename;
        file_put_contents($fullFileName, $data);
        if (filesize($fullFileName)) {
            $result['filesize'] = filesize($fullFileName);
            $result['filename'] = $filename;
            $result['fullpath'] = $fullFileName;
            $result['url'] .= $mediaSubFolder . $filename;
        } else {
            $result['url'] = ' ';
        }

        return $result;
    }

    /**
     * Populates the Temando module fields
     * 
     * @param Object $productObj Magento Product
     * @param Array $productNetsuiteData Netsuite Data
     * @return Object Magento Product
     */
    public function setProductTemandoFields($productObj, $productNetsuiteData)
    {
        $isTemandoInstall = Mage::helper('core')->isModuleEnabled('Ewave_Temando');
        if ($isTemandoInstall) {
            $temandoPackagingOptions = Mage::getModel('temando/system_config_source_shipment_packaging')->getOptions();
            foreach ($temandoPackagingOptions as $key => $label) {
                if (isset($productNetsuiteData['temando_packaging']) && (strtolower($productNetsuiteData['temando_packaging']) == strtolower($label))) {
                    $productObj->setData('temando_packaging', $key);
                }
            }

            if (isset($productNetsuiteData['temando_width'])) {
                $productObj->setData('temando_width', $productNetsuiteData['temando_width']);
            }

            if (isset($productNetsuiteData['temando_length'])) {
                $productObj->setData('temando_length', $productNetsuiteData['temando_length']);
            }

            if (isset($productNetsuiteData['temando_height'])) {
                $productObj->setData('temando_height', $productNetsuiteData['temando_height']);
            }

            if (isset($productNetsuiteData['temando_fragile'])) {
                $productObj->setData('temando_fragile', $productNetsuiteData['temando_fragile']);
            }
        }
        return $productObj;
    }

    /**
     * Adds tags attached to a product
     *
     * Call this function if you want magento tagging feature
     *
     * @param Object $MagentoProduct Magento Product
     * @param array $productNetsuiteData Data array from netsuite
     * @return Object Magento Product
     */
    public function addProductTags($MagentoProduct, $productNetsuiteData)
    {
        $tags = $productNetsuiteData['searchkeywords'];
        $MagentoProduct->setData('cc_searchkeywords', $tags);

        if ($tags) {

            $storeId = Mage::app()->getStore()->getId();
            if (!$storeId && isset($productNetsuiteData['store_id'])) {
                $storeId = $productNetsuiteData['store_id'];
            } else {
                $storeId = 1;
            }
            // Needs the ID for the product
            $MagentoProduct->save();

            // Add tags that does not exist in the system
            $tagCollections = $this->addTags($tags, $storeId);


            // Map tag and product relation
            foreach ($tagCollections as $currTag) {
                $currTag->saveRelation($MagentoProduct->getId(), 1, $storeId);
                $currTag->setAddBasePopularity();
                $currTag->afterCommitCallback();
            }
        }
        return $MagentoProduct;
    }

    /**
     * Adds tags to Magento tags collection
     *
     * This method is called by addProductTags
     *
     * @param string $tags space seperated string
     * @param int $storeId  Magento Store Id
     * @param string $separator Default is space
     * @return array Collection of processed tags
     */
    public function addTags($tags, $storeId, $separator = ' ')
    {

        $tags = str_replace('"', "'", $tags);
        $exploadedTags = $this->_cleanTags($this->_extractTags($tags));
        $tagCollections = array();

        if (empty($exploadedTags)) {
            return array();
        }
        foreach ($exploadedTags as $curTag) {
            if (!$curTag) {
                continue;
            }
            $curTagObj = Mage::getModel('tag/tag');
            $curTagObj->loadByName(trim($curTag));
            if (!$curTagObj->getData()) {
                $curTagObj->setFirstStoreId($storeId)
                        ->setName(trim($curTag))
                        ->setFirstCustomerId(1)
                        ->setStatus($curTagObj->getApprovedStatus())
                        ->save()
                        ->afterCommitCallback();
            }
            $tagCollections[] = $curTagObj;
        }
        return $tagCollections;
    }

    /**
     * Adds product images and exclude some from being display in the product gallary
     *
     * The best way to use this method is to override the setProductImages method
     *
     * Example
     * <pre><code>
     * function setProductImages($productObj,$imagesArray){
     *
     *    $imagesToHide = array(
     *       'custitem_technical_image',
     *        'custitem_back_image,
     *      );
     *
     *     return $this->setProductImagesWithHiddenOnes($ProeductObj,$imagesArray,$imagesToHide);
     * }
     * </code></pre>
     *
     * @param Object $productObj Magento Product object
     * @param array $imagesArray Netsuite array of images
     * @param array $imagesToHide Index array with images netsuite field names
     * @return Object Mangento Product
     */
    public function setProductImagesWithHiddenOnes($productObj, $imagesArray = array(), $imagesToHide = array())
    {
        // First Remove images
        Mage::app()->getStore()->setId(Mage_Core_Model_App::ADMIN_STORE_ID);
        if ($productObj->getId()) {
            $oldProduct = $productObj;
        } else {
            $oldProduct = $this->getProductByNetsuiteId($productObj->getData('cc_netsuite_id'));
        }

        if ($oldProduct) {

            $mediaApi = Mage::getModel("catalog/product_attribute_media_api");
            $items = $mediaApi->items($oldProduct->getId());
            if (is_array($items) && !empty($items)) {
                foreach ($items as $item) {
                    $mediaApi->remove($oldProduct->getId(), $item['file']);
                }
            }
        }

        $productObj->setMediaGallery(array('images' => array(), 'values' => array()));



        // Add product images
        foreach ($imagesArray as $imageIndex => $cur_image) {
            if (!$cur_image['id']) {
                continue;
            }

            $result = $this->getProductMedia($cur_image['id'], $cur_image['name'], $cur_image['type']);

            if ($result['filesize']) {
                if (isset($imagesArray[$imageIndex + 1])) {
                    $productObj->addImageToMediaGallery($result['fullpath'], array(), false, in_array($imagesArray[$imageIndex]['field'], $imagesToHide));
                } else {
                    $productObj->addImageToMediaGallery($result['fullpath'], array('image', 'small_image', 'thumbnail'), false, false);
                }
            }
        }

        $attributes = $productObj->getTypeInstance(true)->getSetAttributes($productObj);
        $gallery = $attributes['media_gallery'];
        $images = $productObj->getMediaGallery('images');
        try {
            foreach ($images as $key => $image) {
                $backend = $gallery->getBackend();
                $backend->updateImage(
                        $productObj, $image['file'], array('label' => $imagesArray[$key]['field'])
                );
            }
        } catch (Exception $e) {
            Mage::log($e->getMessage());
        }
        $productObj->getResource()->saveAttribute($productObj, 'media_gallery');

        return $productObj;
    }

    /**
     * Updates the current Magento Product price and stock level
     * 
     * @param Objec $Product Magento product
     * @param array $productData Data from Netusuite
     * @return Object Magento Product
     */
    public function updateProductStockAndPrice($Product, $productData)
    {
        try {
            $Product->save();
        } catch (Exception $e) {
            Mage::log($e->getMessage());
        }

        // Update price group
        Mage::app()->setCurrentStore(Mage_Core_Model_App::ADMIN_STORE_ID);
        if (isset($productData['price_info']) && $productData['price_info']['group_price']) {

            $Product = Mage::getModel('catalog/product')->load($Product->getId());
            // updating group_price sometimes throws an exception
            // deleting the existing price group solve this issue
            $Product->setData('group_price', array());
            $Product->save();
            try {
                $Product = Mage::getModel('catalog/product')->load($Product->getId());
                $Product->setData('group_price', $productData['price_info']['group_price']);
                $Product->save();
            } catch (Exception $e) {
                Mage::log($e->getMessage());
            }
        }



        // Stock
        $qty = ($productData['quantity']) ? $productData['quantity'] : 0; // Stock can not be zero
        $StockData = Mage::getModel('cataloginventory/stock_item')->loadByProduct($Product->getId());

        if ($StockData->getData()) {
            // For an existing product update the stock
            $StockData->setData('qty', $qty);

            if (isset($productData['set_stock_availablity']) && $productData['set_stock_availablity']) {
                if ($qty >= $StockData->getMinSaleQty()) {
                    $StockData->setData('is_in_stock', 1);
                } else {
                    $StockData->setData('is_in_stock', 0);
                }
            }

            $StockData->setData('manage_stock', 1);
            $StockData->setData('use_config_manage_stock', 1);
            $StockData->save();
        } else {
            // For a new product create a new stock take
            $StockData->setProduct($Product);
            $StockData->setData('qty', $qty);

            if (isset($productData['set_stock_availablity']) && $productData['set_stock_availablity']) {
                if ($qty) {
                    $StockData->setData('is_in_stock', 1);
                } else {
                    $StockData->setData('is_in_stock', 0);
                }
            }

            $StockData->setData('manage_stock', 1);
            $StockData->setData('stock_id', 1);
            $StockData->setData('use_config_manage_stock', 1);
            $StockData->save();
        }


        // Tirer price
        if (isset($productData['price_info']) && $productData['price_info']['tier_price']) {
            $Product = Mage::getModel('catalog/product')->load($Product->getId());
            try {
                $Product->setTierPrice($productData['price_info']['tier_price']);
                $Product->save();
            } catch (Exception $e) {
                Mage::log($e->getMessage());
            }
        }

        return $Product;
    }

    /**
     * Checks inputed tags on the correctness of symbols and split string to array of tags
     *
     * @param string $tagNamesInString
     * @return array
     */
    protected function _extractTags($tagNamesInString)
    {
        return explode("\n", preg_replace("/(\'(.*?)\')|(\s+)/i", "$1\n", $tagNamesInString));
    }

    /**
     * Clears the tag from the separating characters.
     *
     * @param array $tagNamesArr
     * @return array
     */
    protected function _cleanTags(array $tagNamesArr)
    {
        foreach ($tagNamesArr as $key => $tagName) {
            $tagNamesArr[$key] = trim($tagNamesArr[$key], '\'');
            $tagNamesArr[$key] = trim($tagNamesArr[$key]);
            if ($tagNamesArr[$key] == '') {
                unset($tagNamesArr[$key]);
            }
        }
        return $tagNamesArr;
    }

}