<?php

?>
<?php

/**
 * This class retrive the product inventory from NetSuite and
 * update the products inventory in Magento
 *
 */
class CloudCoders_NetSuite_Model_InventoryUpdater extends Mage_Core_Model_Abstract
{

    private $scriptUrl = '';
    private $jsonFile = 'var/log/inventory_json.txt';
    private $indexFile = 'var/log/inventory_index.txt';

    public function __construct()
    {
        $this->scriptUrl = Mage::getStoreConfig('system/netsuite/netsuite_inventory_script');
    }

    /**
     * Retrives the csv file locally or remotely
     * 
     * @return boolean
     */
    public function getJsonFile()
    {

        if ($this->scriptUrl) {

            if (file_exists($this->jsonFile)) {
                $lastIndex = $this->getLastProcessedIndex();
                $csvContent = file_get_contents($this->jsonFile);
                $result = $this->processJsonContent($csvContent);
                if ($lastIndex >= count($result)) {
                    if (file_exists($this->jsonFile)) {
                        @unlink($this->jsonFile);
                    }

                    if (file_exists($this->indexFile)) {
                        @unlink($this->indexFile);
                    }
                    return $this->getJsonFile();
                } else {
                    $csvContent = file_get_contents($this->jsonFile);
                    return $this->processJsonContent($csvContent);
                }
            } else {
                $curl = curl_init();
                curl_setopt($curl, CURLOPT_URL, $this->scriptUrl);
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
                curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
                curl_setopt($curl, CURLOPT_CAINFO, dirname(__FILE__) . "/../certs/BuiltinObjectTokenGoDaddyClass2CA.crt");

                $csvContent = curl_exec($curl);
                file_put_contents($this->jsonFile, $csvContent);
            }
            return $this->processJsonContent($csvContent);
        } else {
            return false;
        }
    }

    /**
     * Attemps to update all products inventory
     * 
     * This method is run once daily by the cron job
     */
    public function updateInventory()
    {

        $this->getJsonFile();
        $SimpleProduct = Mage::getModel('netsuite/simpleproduct');

        if (file_exists($this->jsonFile)) {
            $csvContent = file_get_contents($this->jsonFile);
            $result = $this->processJsonContent($csvContent);

            if ($result && is_array($result)) {
                $counter = $this->getLastProcessedIndex();

                for (; $counter < count($result); $counter++) {

                    $productStock = (isset($result[$counter]['qty'])) ? $result[$counter]['qty'] : 0;
                    $setAvailability = (isset($result[$counter]['set_stock_availability'])) ? $result[$counter]['set_stock_availability'] : 0;
                    $Product = $this->getMagentoProduct($result[$counter]);
                    if ($Product) {
                        $this->doUpdate($Product, $productStock, $setAvailability);

                        $productDataArray = array(
                            'netsuite_record_type' => $result[$counter]['netsuite_record_type'],
                            'item_netsuite_id' => $result[$counter]['netsuite_id'],
                        );
                        $additionalData = array(
                            'stock_update' => true
                        );
                        $SimpleProduct->sendMagentoId($productDataArray, $Product->getId(), $additionalData);
                    }

                    $this->setLastProcessedIndex($counter);
                }
            }

            if (file_exists($this->jsonFile)) {
                @unlink($this->jsonFile);
            }

            if (file_exists($this->indexFile)) {
                @unlink($this->indexFile);
            }
        }
    }

    public function getLastProcessedIndex()
    {
        if (file_exists($this->indexFile)) {
            try {
                $content = file_get_contents($this->indexFile);
                return floatval(trim($content));
            } catch (Exception $ex) {
                return 0;
            }
        } else {
            return 0;
        }
    }

    public function setLastProcessedIndex($index)
    {
        try {
            file_put_contents($this->indexFile, $index);
        } catch (Exception $ex) {
            
        }
    }

    /**
     * Completes the inventory update
     * 
     * This method is run four times daily
     */
    public function completeInventoryUpdate()
    {
        $this->updateInventory();
    }

    /**
     * Returns a Magento product 
     * 
     * @param array $data Stock data from Netsuite
     * @return mix [Object|false]
     */
    private function getMagentoProduct(array $data)
    {
        $productId = (isset($data['magento_id'])) ? trim($data['magento_id']) : 0;
        $productSku = (isset($data['sku'])) ? $data['sku'] : '';
        Mage::app()->getStore()->setId(Mage_Core_Model_App::ADMIN_STORE_ID);
        $Product = Mage::getModel('catalog/product')->load($productId);

        if ($Product && !$Product->getId()) {
            $Product = Mage::getModel('catalog/product')->loadByAttribute('sku', $productSku);
        }

        if ($Product && $Product->getId()) {
            return $Product;
        }
        return false;
    }

    /**
     * Parses the csv file into an array
     * 
     * @param  string $csvContent
     * @return  array
     */
    private function processJsonContent($csvContent)
    {
        $product = substr($csvContent, 0, strpos($csvContent, '[--end--]'));
        return json_decode($product, true);
    }

    /**
     * Does the actual inventory update
     * 
     * @param Object $Product Affected Magento Product
     * @param numeric $qty Stock level
     */
    private function doUpdate($Product, $qty, $setAvailability)
    {
        $StockData = Mage::getModel('cataloginventory/stock_item')->loadByProduct($Product->getId());

        if ($StockData->getData() && $StockData->getData('qty') != $qty) {
            $StockData->setData('qty', $qty);

            if ($setAvailability) {
                if ($qty >= $StockData->getMinSaleQty()) {
                    $StockData->setData('is_in_stock', 1);
                } else {
                    $StockData->setData('is_in_stock', 0);
                }
            }

            $StockData->setData('manage_stock', 1);
            $StockData->setData('use_config_manage_stock', 1);
            $StockData->save();
        } else {
            
        }
    }

}