<?php

?>
<?php

/**
 * This class handles the sales order and customer save event
 * trigger by magento
 *
 */
class CloudCoders_NetSuite_Model_Observer
{

    const DEBUG_TO_SESSION = false;
    const DEBUG_TO_LOG = false;
    const SEND_IMMEDIATELY = true;

    private static $activeCustomers = array();
    private static $activeOrder = array();

    public function __construct()
    {
        
    }

    /**
     * Handles customer save event
     *
     * @param object $observer Magento Observer
     * @return \CloudCoders_NetSuite_Model_Observer
     */
    public function trigger_customer_update($observer)
    {
        // Disable customer syncing if the customer was created
        // from netsuite
        if (Mage::app()->getStore()->getConfig('cloudcoders_disable_customer_event') == true) {
            return;
        }
        $this->debug_message('trigger_customer_update');

        $event = $observer->getEvent();
        $customer = $event->getCustomer();

        $customer_id = $customer->getEntityId();
        $this->debug_message('customer_id=' . $customer_id);

        if (isset(self::$activeCustomers[$customer_id])) {
            return $this;
        }

        //Check whether to send now or later
        $automated = Mage::getStoreConfig('system/netsuite/netsuite_automate_salesorder');
        if (self::SEND_IMMEDIATELY && $automated) {
            $helper = Mage::helper('netsuite/sender');
            if (!$helper->sendCustomerToNetsuite($customer_id)) {
                $this->debug_message($helper->getErrorMessage());
            } else {
                $this->debug_message('Customer updated successfully');
            }
        } else {
            $queueEntry = Mage::getModel('netsuite/queue');
            $queueEntry->setTypeId($customer->getEntityTypeId());
            $queueEntry->setEntityId($customer_id);
            $queueEntry->save();

            $this->debug_message('Saved with queue_id of ' . $queueEntry->getQueueId());
        }

        self::$activeCustomers[$customer_id] = true;

        return $this;
    }

    /**
     * Handles sales order save event
     * @param object $observer Magento Observer
     * @return \CloudCoders_NetSuite_Model_Observer
     */
    public function trigger_sales_order_update($observer)
    {
        $this->debug_message('trigger_sales_order_update');

        $event = $observer->getEvent();
        $order = $event->getOrder();

        $order_id = $order->getEntityId();
        $this->debug_message('order_id=' . $order_id);

        $orderStatus = $order->getStatus();
        $this->debug_message('orderStatus=' . $orderStatus);

        if (isset(self::$activeOrder[$order_id])) {
            return $this;
        }

        // Disable customer syncing if the customer was created
        // from netsuite
        if (Mage::app()->getStore()->getConfig('cloudcoders_disable_order_event') == true) {
            self::$activeOrder[$order_id] = true;
            return;
        }


        if ($orderStatus != null && $orderStatus != 'complete') {
            self::$activeOrder[$order_id] = true;
            Mage::app()->getStore()->setConfig('cloudcoders_disable_order_event', true);
            $automated = Mage::getStoreConfig('system/netsuite/netsuite_automate_salesorder');
            if (self::SEND_IMMEDIATELY && $automated) {
                $helper = Mage::helper('netsuite/sender');

                $this->debug_message('Sending sales order ' . $order_id);

                if (!$helper->sendOrderToNetsuite($order_id)) {
                    $this->debug_message($helper->getErrorMessage());
                } else {
                    $this->debug_message('Order updated successfully');
                }
            } else {
                $this->debug_message('saving sales order ' . $order_id);
                $queueEntry = Mage::getModel('netsuite/queue');
                $queueEntry->setTypeId(Mage::getModel('eav/entity')->setType('order')->getEntityType()->getEntityTypeId());
                $queueEntry->setEntityId($order_id);
                $queueEntry->save();

                $this->debug_message('Saved with queue_id of ' . $queueEntry->getQueueId());
            }
            Mage::app()->getStore()->setConfig('cloudcoders_disable_order_event', false);
        }

        return $this;
    }

    public function trigger_sales_order_before_save($observer)
    {
        $CustomeOrder = Mage::helper('netsuite/factory')->newOrder(0);
        $CustomeOrder->beforeSave($observer);
    }

    /**
     * Stores debug message either to session or database
     *
     * @param string $pMessage
     */
    private function debug_message($pMessage)
    {
        if (self::DEBUG_TO_SESSION) {
            Mage::getSingleton('core/session')->addNotice($pMessage);
        }
        if (self::DEBUG_TO_LOG) {
            Mage::log($pMessage);
        }
    }

}