<?php

?>
<?php

/**
 * This Class creates a simple product from data sent from netsuite
 *
 */
class CloudCoders_Netsuite_Model_Simpleproduct extends Mage_Core_Model_Abstract
{

    const IN_QUEUE = 1;
    const PROCESSING = 2;
    const COMPLETE = 3;

    protected $NetSuiteFileType;
    private $netsuiteItemMagentoIdUpdateUrl = false;
    protected $mediaUrl;
    protected $productNetsuiteData = false;
    private $isBulkImport = false;

    public function _construct()
    {
        parent::_construct();
        $this->_init('netsuite/simpleproduct');
        $this->NetSuiteFileType = Mage::helper('netsuite/netsuiteFileType');
        $this->netsuiteItemMagentoIdUpdateUrl = Mage::getStoreConfig('system/netsuite/netsuite_item_id_script');
        $this->mediaUrl = Mage::getStoreConfig('system/netsuite/netsuite_media_script') . '&custom_id=';
    }

    /**
     * Returns the item data for the current magento product
     *
     * @return array
     */
    public function getItemNetsuiteData()
    {
        return $this->productNetsuiteData;
    }

    /**
     * Sets the current product netsuite data
     *
     * This function is called by processQueue for
     * each item data
     *
     * @param array $productNetsuiteData Data from netsuite
     * @return \CloudCoders_Netsuite_Model_Simpleproduct
     */
    public function setItemNetsuiteData($productNetsuiteData)
    {
        $this->productNetsuiteData = $productNetsuiteData;
        return $this;
    }

    /**
     * Added or edit a product
     *
     * @param array $productData Netsuite item data
     * @return array execution result
     */
    public function addProduct($productData)
    {
        $result = array(
            'success' => false,
        );

        if (!isset($productData['item_netsuite_id'])) {
            return;
        }

        // First try to update the item
        $Product = $this->updateProduct($productData['item_netsuite_id']);

        // Get any customised product attribute
        $Product = $this->getNesuiteCustomProduct($Product, $productData);

        $result['success'] = true;
        $result['message'] = 'Netsuite item was added successfully';
        $result['result'] = array(
            'magento_id' => $Product->getId(),
        );

        // Sends the product new or existing magento ID to
        // Netsuite
        $this->sendMagentoId($productData, $Product->getId());

        return $result;
    }

    /**
     * Deletes an existing product from magento
     *
     * @param array $productData
     * @param bool $isNetsuiteId
     */
    public function deleteProduct($productData, $isNetsuiteId = false)
    {
        $OldProduct = false;
        if ($isNetsuiteId) {
            $id = $productData['item_netsuite_id'];
            $magento_id = $productData['magento_id'];

            $OldProduct = $this->getProductByNetsuiteId($id);
        }

        if ($OldProduct && ($OldProduct->getId() == $magento_id)) {
            /**
             * Disable the product instead of deleting it
             * Leaving the commented out code for reference only
             */
            // Mage::register('isSecureArea', true);
            // $OldProduct->delete();

            $OldProduct->setStatus(2); // Disable the product
            $OldProduct->save();
        }
    }

    /**
     * Returns a product for updating
     *
     * @param int $productNetsuiteId Netsuite item ID
     * @return \Mage_Catalog_Model_Product
     */
    public function updateProduct($productNetsuiteId)
    {
        $Product = $this->getProductByNetsuiteId($productNetsuiteId);
        if ($Product) {
            return $Product;
        }

        return new Mage_Catalog_Model_Product();
    }

    /**
     * Returns a magento product
     *
     * This function uses
     * @param type $netsuiteId
     * @return type
     */
    public function getProductByNetsuiteId($netsuiteId)
    {

        Mage::app()->getStore()->setId(Mage_Core_Model_App::ADMIN_STORE_ID);
        $product = Mage::getModel('catalog/product')->loadByAttribute('cc_netsuite_id', $netsuiteId);
        $netsuiteData = $this->getItemNetsuiteData();

        if (!$product && $netsuiteData && isset($netsuiteData['magento_id']) && $netsuiteData['magento_id'] > 0) {

            $MagentoProduct = Mage::getModel('catalog/product')->load($netsuiteData['magento_id']);
            if ($MagentoProduct && $MagentoProduct->getData('cc_netsuite_id') == $netsuiteId) {
                return $MagentoProduct;
            }
        } elseif (!$product && $netsuiteData && isset($netsuiteData['sku'])) {
            $MagentoProduct = Mage::getModel('catalog/product')->loadByAttribute('sku', $netsuiteData['sku']);
            if ($MagentoProduct && $MagentoProduct->getDate('cc_netsuite_id') == $netsuiteId) {
                return $MagentoProduct;
            }
        }
        return $product;
    }

    /**
     * Updates the specified magento product's item in netsuite
     *
     * @param array $productDataArray product netsuite data
     * @param int $productMangentoId product magento ID
     */
    public function sendMagentoId($productDataArray, $productMangentoId, $additionalData = array())
    {
        $url = $this->netsuiteItemMagentoIdUpdateUrl;

        // fields
        $url .= '&record_type=' . $productDataArray['netsuite_record_type'];
        $url .= '&item_id=' . $productDataArray['item_netsuite_id'];
        $url .= '&magento_id=' . $productMangentoId;

        if ($additionalData) {
            foreach ($additionalData as $key => $value) {
                $url .= '&' . $key . '=' . $value;
            }
        }

        if ($this->isBulkImport) {
            $url .= '&display_in_magento=1';
        }

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_exec($ch);
    }

    /**
     * Returns current product after updating it's attributes
     *
     * @param object $MagentoProduct Magento standard product object
     * @param array $productData Magento
     * @return type
     */
    public function getNesuiteCustomProduct($MagentoProduct, $productData)
    {

        $defaultProductFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Product.php';
        require_once $defaultProductFile;

        if (isset($productData['company_id'])) {
            $customProductFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'Custom' . DIRECTORY_SEPARATOR . 'Product_' . $productData['company_id'] . '.php';
            $className = 'Product_' . $productData['company_id'];
            if (file_exists($customProductFile)) {
                require_once $customProductFile;
                $CustomProduct = new $className($MagentoProduct, $productData);
                return $CustomProduct->getBuildProduct();
            }
        }

        $CustomProduct = new Product($MagentoProduct, $productData);
        return $CustomProduct->getBuildProduct();
    }

    /**
     * Returns a Magento Category By it's Netsuite Id
     *
     * @param int $categoryNetsuiteId
     * @return Object Magento Category
     */
    public function getProductCategoryByNetsuiteId($categoryNetsuiteId)
    {
        $Category = Mage::getModel('catalog/category')->loadByAttribute('cc_netsuite_id', $categoryNetsuiteId);
        return $Category;
    }

    public function setIsBulkImport($isBulkImport = true)
    {
        $this->isBulkImport = $isBulkImport;
    }

}