<?php

?>
<?php

/**
 * Displays the various configurations needed by the integration module
 *
 */
class CloudCoders_NetSuite_ConfigurationController extends Mage_Core_Controller_front_Action
{

    /**
     * Returns the full list of enabled shipping methods
     *
     * @return string JSON String
     */
    public function getShippingMethodAction()
    {
        return json_encode($this->getShippingMethod());
    }

    /**
     * Displays all the configuration parameters
     *
     */
    public function getConfigAction()
    {
        $config = array();

        $config['shipping_methods'] = $this->getShippingMethod();
        $config['payment_methods'] = $this->getPaymentMethod();
        $config['customer_groups'] = $this->getGroups();
        $config['stores'] = $this->getStores();

        print(json_encode($config));
    }

    /**
     * Returns the various customer groups
     *
     * @return string JSON string
     */
    public function getGroupsAction()
    {
        return json_encode($this->getGroups());
    }

    /**
     * Returns all the stores Id
     *
     * @return string JSON string
     */
    public function getStoresAction()
    {
        return json_encode($this->getStores());
    }

    /**
     * Returns the full list of customer groups
     *
     * @return array List of groups
     */
    private function getGroups()
    {
        $groups = Mage::getModel('customer/group');
        $groupCollection = array();
        foreach ($groups->getCollection() as $key) {
            $groupCollection[] = array(
                'code' => $key->getId(),
                'group' => $key->getCode(),
            );
        }
        return $groupCollection;
    }

    /**
     * Returns the list of enabled shipping methods
     *
     * @return array
     */
    private function getShippingMethod()
    {
        $carries = array();
        $existingMethod = array();
        $stores = $this->getStores();
        
        foreach ($stores as $currentStore) {
            $allActiveShippingMethods = Mage::getSingleton('shipping/config')->getActiveCarriers($currentStore['code']);
            $methods = $allActiveShippingMethods;
            foreach ($methods as $_ccode => $_carrier) {
                if ($_methods = $_carrier->getAllowedMethods()) {
                    if (!$_title = Mage::getStoreConfig("carriers/$_ccode/title")) {
                        $_title = $_ccode;
                    }
                    foreach ($_methods as $_mcode => $_method) {
                        $_code = $_ccode . '_' . $_mcode;
                        if (isset($existingMethod[$_ccode])) {
                            continue;
                        }
                        $existingMethod[$_ccode] = true;

                        $carries[] = array(
                            'code' => $_ccode,
                            'carrier' => $_title
                        );
                    }
                }
            }
        }

        return $carries;
    }

    /**
     * Returns  the list of enabled payment methods
     *
     * @return array
     */
    private function getPaymentMethod()
    {
        $paymentMehtod = array();

        $allActivePaymentMethods = Mage::getModel('payment/config')->getActiveMethods();
        foreach ($allActivePaymentMethods as $curMethod) {
            $paymentMehtod[] = array(
                'code' => $curMethod->getCode(),
                'method' => $curMethod->getTitle(),
            );
        }
        return $paymentMehtod;
    }

    /**
     * Returns the list of the stores Ids
     *
     * @return array
     */
    private function getStores()
    {
        $allStores = Mage::app()->getStores();
        $stores = array();
        foreach ($allStores as $currentStore) {
            $stores[] = array(
                'id' => $currentStore->getId(),
                'code' => $currentStore->getCode(),
                'name' => $currentStore->getName(),
            );
        }
        return $stores;
    }

}